/*-
 * Copyright (c) 2011-2012       Scott Ringwelski <sgringwe@mtu.edu>
 *
 * Originally Written by Scott Ringwelski for BeatBox Music Player
 * BeatBox Music Player: http://www.launchpad.net/beat-box
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Xml;

public class LastFM.SimilarMedias : Object {
	BeatBox.LibraryManager _lm;
	BeatBox.Media _base;
	bool working;
	
	Gee.LinkedList<BeatBox.Media> similar;
	
	public signal void similar_retrieved( Gee.LinkedList<int> similarIDs, Gee.LinkedList<BeatBox.Media> similarDont);
	
	public class SimilarMedias(BeatBox.LibraryManager lm) {
		_lm = lm;
		working = false;
	}
	
	public virtual void queryForSimilar(BeatBox.Media s) {
		_base = s;
		
		if(!working) {
			working = true;
			
			try {
				Thread.create<void*>(similar_thread_function, false);
			}
			catch(GLib.ThreadError err) {
				warning("ERROR: Could not create similar thread: %s \n", err.message);
			}
		}
	}
	
	public void* similar_thread_function () {	
		similar = new Gee.LinkedList<BeatBox.Media>();
		var similarIDs = new Gee.LinkedList<int>();
		var similarDont = new Gee.LinkedList<BeatBox.Media>();
		
		getSimilarTracks(_base.title, _base.artist);
		_lm.medias_from_name(similar, ref similarIDs, ref similarDont);
		similarIDs.offer_head(_base.rowid);
		
		Idle.add( () => {
			similar_retrieved(similarIDs, similarDont);
			return false;
		});
		
		working = false;
		
		return null;	
    }
	
	/** Gets similar medias
	 * @param artist The artist of media to get similar to
	 * @param title The title of media to get similar to
	 * @return The medias that are similar
	 */
	public void getSimilarTracks(string title, string artist) {
		var artist_fixed = LastFM.Core.fix_for_url(artist);
		var title_fixed =  LastFM.Core.fix_for_url(title);
		var url = "http://ws.audioscrobbler.com/2.0/?method=track.getsimilar&artist=" + artist_fixed + "&track=" + title_fixed + "&api_key=" + LastFM.Core.api;
		
		Soup.SessionSync session = new Soup.SessionSync();
		Soup.Message message = new Soup.Message ("GET", url);
		
		session.timeout = 30;// after 30 seconds, give up
		
		/* send the HTTP request */
		session.send_message(message);
		
		Xml.Doc* doc = Xml.Parser.parse_memory((string)message.response_body.data, (int)message.response_body.length);
		
		if(doc == null)
			GLib.message("Could not load similar artist information for %s by %s\n", title, artist);
		else if(doc->get_root_element() == null)
			GLib.message("Oddly, similar artist information was invalid\n");
		else {
			GLib.message("Getting similar tracks with %s... \n", url);
			
			parse_similar_nodes(doc->get_root_element(), "");
		}
		
		delete doc;
	}
	
	void parse_similar_nodes(Xml.Node* node, string parent) {
		Xml.Node* iter;
		for (iter = node->children; iter != null; iter = iter->next) {
			
            if (iter->type != ElementType.ELEMENT_NODE) {
                continue;
            }

            string node_name = iter->name;
            
            if(parent == "similartracks") {
				if(node_name == "track") {
					BeatBox.Media m = new BeatBox.Media("");
					m.isTemporary = true;
					m.album = "";
					
					Xml.Node* track_iter;
					for(track_iter = iter->children; track_iter != null; track_iter = track_iter->next) {
						string track_node_name = track_iter->name;
						string track_node_content = track_iter->get_content ();
						
						if(track_node_name == "name")
							m.title = track_node_content;
						// Disable duration since they are incorrect from last.fm
						/*else if(track_node_name == "duration")
							m.length = int.parse(track_node_content)/1000;*/
						else if(track_node_name == "url")
							m.lastfm_url = track_node_content;
						else if(track_node_name == "artist") {
							Xml.Node* artist_iter;
							for(artist_iter = track_iter->children; artist_iter != null; artist_iter = artist_iter->next) {
								string artist_node_name = artist_iter->name;
								
								if(artist_node_name == "name") {
									m.artist = artist_iter->get_content();
									m.album_artist = m.artist;
									break;
								}
							}
						}
					}
					
					similar.add(m);
				}
			}
			
			parse_similar_nodes(iter, parent+node_name);
		}
		
		delete iter;
	}
}
