/*-
 * Copyright (c) 2011-2012       Scott Ringwelski <sgringwe@mtu.edu>
 *
 * Originally Written by Scott Ringwelski for BeatBox Music Player
 * BeatBox Music Player: http://www.launchpad.net/beat-box
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gee;
using Gtk;

public class BeatBox.NowPlayingView : ScrolledWindow {
	LibraryManager lm;
	LibraryWindow lw;
	LyricFetcher lyric_fetcher;
	static CssProvider style_provider;
	
	// Switches b/w video and song info
	Gtk.Notebook notebook;
	
	// Holds song info
	Box box;
	
	// Top bar
	Button loveMedia;
	Button banMedia;
	RatingWidget rating;
	
	HBox summary_box;
	Gtk.Image coverArt;
	Label meta_title;
	Label meta_artist;
	Label meta_album;
	Label meta_year;
	StyledBox lyrics_holder;
	Label lyrics;
	StyledBox album_songs_holder;
	MusicList album_songs;
	Label summary_text;
	
	VBox artist_box;
	Label artist_section_label;
	Label top_tracks_label;
	Label similar_artists;
	Label artist_tags;
	//Label top_albums_label;
	StyledBox artist_top_songs_holder;
	MusicList artist_top_songs;
	StyledBox artist_top_albums_holder;
	ExternalAlbumGrid artist_top_albums;
	StyledArtistImages artist_images;
	
	//Granite.Widgets.StaticNotebook summary_notebook;
	//Label track_switcher;
	//Label album_switcher;
	//Label artist_switcher;
	//Label track_info;
	//Label album_info;
	//Label artist_info;
	
	public DrawingArea videoArea;
	
	private const string WIDGET_STYLESHEET = """
        .NowPlayingView {
			background-image: -gtk-gradient (linear,
                                               left top, left bottom,
                                               from (shade (#ffffff, 1.05)),
                                               to (#ffffff));
            border: 0;
        }
        
        .AlbumSongList {
			/*background-color: rgba(0, 0, 0, 0);*/
			padding: 0;
			/*box-shadow: 10px 10px rgba(0, 0, 0, 0.4);*/
		}
		
		.AlbumSongList row:selected {
			/*background-image: -gtk-gradient (linear,
			                                 left top,
			                                 left bottom,
			                                 from (shade (@selected_bg_color, 1.20)),
			                                 to (shade (@selected_bg_color, 0.98)));*/
		}
		
		.gray {
			background-image: -gtk-gradient (linear,
                                             left top, 
                                             left bottom,
                                             from (shade (#d5d3d1, 1.00)),
                                             to (shade (#d5d3d1, 0.95)));
		}
		
		.black {
			background-image: -gtk-gradient (linear,
                                               left top, left bottom,
                                               from (shade (#383838, 1.05)),
                                               to (#383838));
		}
		
		.white {
			background-image: -gtk-gradient (linear,
                                               left top, left bottom,
                                               from (shade (#f8f8f8, 1.05)),
                                               to (#f8f8f8));
		}
		
		.pure_white {
			background-color: #fff;
			background-image: none;
		}
		
		.white_text {
			color: shade(#f0f0f0, 1.01);
			text-shadow: 1 1 0 alpha(#f8f8f8, 0.3);
		}
		
    """;
	
	public NowPlayingView(LibraryManager lm, LibraryWindow lw) {
		this.lm = lm;
		this.lw = lw;
		lyric_fetcher = new LyricFetcher();
		
        this.set_policy(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
        
		notebook = new Gtk.Notebook();
		notebook.show_tabs = false;
		notebook.show_border = false;
		
		videoArea = new DrawingArea();
		notebook.append_page (videoArea);
		
		box = new VBox(false, 0);
		notebook.append_page(box);
		
		// Add the notebook to the viewport
		if(style_provider == null) {
			style_provider = new CssProvider();
			try  {
				style_provider.load_from_data (WIDGET_STYLESHEET, -1);
			} catch (Error e) {
				stderr.printf ("\nNowPlayingView: Couldn't load style provider.\n");
			}
		}
		
		var event_box = new EventBox();
        event_box.get_style_context().add_class("NowPlayingView");
		event_box.get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		event_box.add(notebook);
		Viewport vp = new Viewport(null, null);
		vp.set_shadow_type(ShadowType.NONE);
		vp.add(event_box);
		add(vp);
		
		add_summary();
		add_artist_section();
		
		hide_video_mode();
		
		lm.media_played.connect(media_played);
		lm.playback_stopped.connect(playback_stopped);
		lm.medias_updated.connect(medias_updated);
		lyric_fetcher.lyrics_fetched.connect(lyrics_fetched);
		size_allocate.connect(size_allocate_signal);
		lm.lfm.track_fetched.connect(update_summary);
		lm.lfm.album_fetched.connect(update_summary);
		lm.lfm.album_fetched.connect(update_album_songs);
		lm.lfm.artist_fetched.connect(update_summary);
		lm.lfm.artist_fetched.connect(update_artist_section);
		lm.lfm.top_artist_songs_retrieved.connect(top_artist_songs_retrieved);
		lm.lfm.top_artist_albums_retrieved.connect(top_artist_albums_retrieved);
		
		show_all();
		set_no_show_all(true);
		
		// Start out w/ correct visibilities
		summary_box.set_visible(lm.media_info.media != null);
		artist_box.set_visible(lm.media_info.media != null && lm.media_info.media.mediatype == MediaType.SONG);
	}
	
	void add_summary() {
		summary_box = new HBox(false, 0);
		
		// First add the metadata stuff coverart, song, artist, album
		var left_side = new VBox(false, 0);
		var metadata = new HBox(false, 0);
		var meta_labels = new VBox(false, 0);
		
		coverArt = new Gtk.Image();
		meta_title = new Label("");
		meta_artist = new Label("");
		meta_album = new Label("");
		HBox rate_box = new HBox(false, 0);
		loveMedia = new Button();
		banMedia = new Button();
		rating = new RatingWidget(null, true, IconSize.MENU);
		meta_year = new Label("");
		summary_text = new Label("");
		lyrics = new Label("");
		
		rate_box.pack_start(rating, false, false, 0);
		rate_box.pack_start(loveMedia, false, false, 0);
		rate_box.pack_start(banMedia, false, false, 0);
		
		metadata.pack_start(coverArt, false, false, 0);
		metadata.pack_start(wrap_alignment(meta_labels, 0, 0, 0, 6), false, false, 0);
		meta_labels.pack_start(meta_title, false, false, 0);
		meta_labels.pack_start(meta_artist, false, false, 0);
		meta_labels.pack_start(meta_album, false, false, 0);
		meta_labels.pack_start(meta_year, false, false, 0);
		meta_labels.pack_start(rate_box, false, false, 0);
		
		coverArt.xalign = 0.0f;
		meta_title.xalign = 0.0f;
		meta_artist.xalign = 0.0f;
		meta_album.xalign = 0.0f;
		meta_year.xalign = 0.0f;
		summary_text.xalign = 0.0f;
		lyrics.xalign = 0.0f;
		coverArt.yalign = 0.0f;
		meta_title.yalign = 0.0f;
		meta_artist.yalign = 0.0f;
		meta_album.yalign = 0.0f;
		meta_year.yalign = 0.0f;
		summary_text.yalign = 0.0f;
		lyrics.yalign = 0.0f;
		meta_title.set_line_wrap(true);
		meta_artist.set_line_wrap(true);
		meta_album.set_line_wrap(true);
		summary_text.set_line_wrap(true);
		lyrics.set_line_wrap(true);
		coverArt.set_size_request (Icons.ALBUM_VIEW_IMAGE_SIZE, Icons.ALBUM_VIEW_IMAGE_SIZE);
		loveMedia.set_no_show_all(true);
		banMedia.set_no_show_all(true);
		loveMedia.relief = ReliefStyle.NONE;
		banMedia.relief = ReliefStyle.NONE;
		var lastfm_love_icon = Icons.LASTFM_LOVE.render (IconSize.MENU);
		var lastfm_ban_icon = Icons.LASTFM_BAN.render (IconSize.MENU);
		loveMedia.set_image(new Image.from_pixbuf(lastfm_love_icon));
		banMedia.set_image(new Image.from_pixbuf(lastfm_ban_icon));
		
		rating.rating_changed.connect(rating_changed);
		loveMedia.clicked.connect(loveButtonClicked);
		banMedia.clicked.connect(banButtonClicked);
		lm.lfm.logged_in.connect(logged_in_to_lastfm);
		
		drag_dest_set(this, DestDefaults.ALL, {}, Gdk.DragAction.MOVE);
		Gtk.drag_dest_add_uri_targets(this);
		drag_motion.connect(drag_motion_signal);
		drag_leave.connect(drag_leave_signal);
		drag_data_received.connect(drag_received);
		
		// Now the album list on left side under metadata
		album_songs_holder = new StyledBox("Album Songs", "white");
		var tvs = new TreeViewSetup(MusicList.MusicColumn.TRACK, SortType.ASCENDING, ViewWrapper.Hint.NOW_PLAYING);
		album_songs = new MusicList(lm, tvs, -1);
		album_songs.is_mixed = true;
		album_songs.set_sort_column_id(tvs.sort_column_id, tvs.sort_direction);
		album_songs.get_style_context().add_class("AlbumSongList");
		album_songs.get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		var scroll = new ScrolledWindow(null, null);
		scroll.add(album_songs);
		scroll.set_policy(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
		scroll.set_size_request(-1, 160);
		album_songs_holder.set_widget(scroll);
		
		// Summaries go under album list
		//summary_holder = new StyledBox("", "gray");
		var summary_scroll = new ScrolledWindow(null, null);
		var summary_eb = new EventBox();
		summary_eb.get_style_context().add_class("pure_white");
		summary_eb.get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		summary_eb.add(summary_text);
		//summary_scroll.add_with_viewport(summary_eb);
		summary_scroll.set_policy(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
		summary_scroll.set_size_request(-1, 130);
		//summary_holder.set_widget(summary_scroll);
		
		// And lastly the lyrics box
		lyrics_holder = new StyledBox("", "black");
		lyrics.get_style_context().add_class("white_text");
		lyrics.get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		var lyrics_scroll = new ScrolledWindow(null, null);
		var lyrics_eb = new EventBox();
		lyrics_eb.get_style_context().add_class("black");
		lyrics_eb.get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		lyrics_eb.add(lyrics);
		lyrics_scroll.add_with_viewport(lyrics_eb);
		lyrics_scroll.set_policy(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
		lyrics_holder.set_size_request(300, -1);
		lyrics_holder.set_widget(lyrics_scroll);
		
		left_side.pack_start(metadata, false, true, 0);
		left_side.pack_start(wrap_alignment(summary_eb, 6, 0, 0, 0), true, true, 0);
		left_side.pack_start(wrap_alignment(album_songs_holder, 6, 0, 0, 0), true, true, 0);
		
		summary_box.pack_start(wrap_alignment(left_side, 0, 0, 0, 0), true, true, 0);
		summary_box.pack_end(wrap_alignment(lyrics_holder, 0, 0, 0, 12), false, true, 0);
		
		box.pack_start(wrap_alignment(summary_box, 18, 18, 12, 18), false, true, 0);
	}
	
	void add_artist_section() {
		artist_box = new VBox(false, 0);
		var hbox = new HBox(false, 0);
		var left_side = new VBox(false, 0);
		var right_side = new VBox(false, 0);
		
		artist_section_label = new Label("");
		top_tracks_label = new Label("");
		similar_artists = new Label("");
		artist_tags = new Label("");
		
		artist_section_label.set_markup("<span size=\"x-large\"><b>" + _("Artist") + "</b></span>");
		artist_section_label.xalign = 0.0f;
		
		artist_top_songs_holder = new StyledBox("Top Songs", "white");
		var tvs = new TreeViewSetup(MusicList.MusicColumn.NUMBER, SortType.ASCENDING, ViewWrapper.Hint.NOW_PLAYING);
		tvs.set_column_visible(MusicList.MusicColumn.NUMBER, true);
		tvs.set_column_visible(MusicList.MusicColumn.TRACK, false);
		artist_top_songs = new MusicList(lm, tvs, -1);
		artist_top_songs.is_mixed = true;
		artist_top_songs.set_sort_column_id(tvs.sort_column_id, tvs.sort_direction);
		artist_top_songs.get_style_context().add_class("AlbumSongList");
		artist_top_songs.get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		var scroll = new ScrolledWindow(null, null);
		scroll.set_policy(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
		scroll.add(artist_top_songs);
		artist_top_songs_holder.set_widget(scroll);
		artist_top_songs_holder.set_size_request(-1, 180);
		
		artist_top_albums_holder = new StyledBox("Top albums", "white");
		artist_top_albums = new ExternalAlbumGrid(lm);
		artist_top_albums.column_spacing = 0;
		artist_top_albums.item_padding = 0;
		artist_top_albums.spacing = 0;
		artist_top_albums.get_style_context().add_class("AlbumSongList");
		artist_top_albums.get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		var scroll2 = new ScrolledWindow(null, null);
		scroll2.add(artist_top_albums);
		scroll2.set_policy(PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
		artist_top_albums_holder.set_widget(scroll2);
		artist_top_albums_holder.set_size_request(175, 400);
		artist_top_albums.item_chosen.connect(artist_top_albums_chosen);
		
		artist_images = new StyledArtistImages(get_style_context());
		var scroll3 = new ScrolledWindow(null, null);
		scroll3.add_with_viewport(artist_images);
		scroll3.set_policy(PolicyType.AUTOMATIC, PolicyType.NEVER);
		
		top_tracks_label.xalign = 0.0f;
		similar_artists.xalign = 0.0f;
		artist_tags.xalign = 0.0f;
		top_tracks_label.yalign = 0.0f;
		similar_artists.yalign = 0.0f;
		artist_tags.yalign = 0.0f;
		top_tracks_label.set_line_wrap(true);
		similar_artists.set_line_wrap(true);
		artist_tags.set_line_wrap(true);
		top_tracks_label.set_markup(_("<b>%s</b>").printf(_("Top Tracks")));
		
		left_side.pack_start(artist_top_albums_holder, true, true, 0);
		
		right_side.pack_start(artist_top_songs_holder, false, true, 0);
		right_side.pack_start(wrap_alignment(similar_artists, 6, 0, 0, 1), false, true, 0);
		right_side.pack_start(wrap_alignment(artist_tags, 6, 0, 0, 1), false, true, 0);
		right_side.pack_end(scroll3, false, true, 0);
		
		hbox.pack_start(left_side, false, true, 0);
		hbox.pack_end(wrap_alignment(right_side, 0, 0, 0, 12), true, true, 0);
		
		//artist_box.set_widget(hbox);
		//artist_box.set_size_request(-1, 400);
		
		artist_box.pack_start(wrap_alignment(artist_section_label, 6, 0, 6, 0), false, true, 0);
		artist_box.pack_start(hbox, false, true, 0);
		
		box.pack_start(wrap_alignment(artist_box, 0, 18, 12, 18), false, true, 0);
	}
	
	static Gtk.Alignment wrap_alignment (Gtk.Widget widget, int top, int right, int bottom, int left) {
		var alignment = new Gtk.Alignment(0.0f, 0.0f, 1.0f, 1.0f);
		alignment.top_padding = top;
		alignment.right_padding = right;
		alignment.bottom_padding = bottom;
		alignment.left_padding = left;
		
		alignment.add(widget);
		return alignment;
	}
	
	public void logged_in_to_lastfm() {
		update_sensitivities();
	}
	
	void media_played(Media m, Media? old) {
		update_contents();
		
		summary_box.set_visible(true);
		lyric_fetcher.fetch_lyrics(m.artist, m.album_artist, m.title);
		
		if(old == null || old.album_artist != m.album_artist) {
			artist_images.clear();
			artist_images.fetch_images(m.album_artist);
			lm.lfm.fetchTopArtistSongs();
			lm.lfm.fetchTopArtistAlbums();
			artist_box.set_visible(false);
			artist_top_songs_holder.set_visible(false);
			artist_top_albums_holder.set_visible(false);
		}
	}
	
	void playback_stopped(int was_playing) {
		summary_box.set_visible(false);
		artist_box.set_visible(false);
	}
	
	void lyrics_fetched(Lyrics l) {
		Media m = lm.media_info.media;
		if(m != null && m.title == l.title) {
			m.lyrics = l.content;
			update_lyrics();
		}
	}
	
	void medias_updated(Collection<int> ids) {
		update_contents();
	}
	
	void rating_changed(int new_rating) {
		lm.media_info.media.rating = new_rating;
		lm.update_media(lm.media_info.media, false, true, true);
	}
	
	void top_artist_songs_retrieved(HashTable<int, Media> songs) {
		artist_top_songs.set_table(songs);
		artist_top_songs_holder.set_visible(songs.size() > 0);
		
		update_sensitivities();
	}
	
	void top_artist_albums_retrieved(HashTable<int, ExternalAlbum> albums) {
		artist_top_albums.set_table(albums);
		artist_top_albums_holder.set_visible(albums.size() > 0);
		
		update_sensitivities();
	}
	
	void artist_top_albums_chosen(GLib.Object o) {
		ExternalAlbum a = (ExternalAlbum)o;
		
		try {
			GLib.AppInfo.launch_default_for_uri (a.url, null);
		}
		catch(Error err) {
			stdout.printf("Couldn't open the similar media's last fm page: %s\n", err.message);
		}
	}
	
	void loveButtonClicked() {
		lm.lfm.loveTrack(lm.media_info.media.title, lm.media_info.media.artist);
	}
	
	void banButtonClicked() {
		lm.lfm.banTrack(lm.media_info.media.title, lm.media_info.media.artist);
		
		// Clearly we should skip this song...
		lw.nextClicked();
	}

	void mediaEditorSaved(LinkedList<int> medias) {
		LinkedList<Media> toUpdate = new LinkedList<Media>();
		foreach(int i in medias)
			toUpdate.add(lm.media_from_id(i));

		// could have edited rating, so record_time is true
		lm.update_medias(toUpdate, true, true, true);
	}
	
	bool is_valid_image_type(string type) {
		var typeDown = type.down();
		
		return (typeDown.has_suffix(".jpg") || typeDown.has_suffix(".jpeg") ||
				typeDown.has_suffix(".png"));
	}
	
	bool drag_motion_signal(Gdk.DragContext context, int x, int y, uint time) {
		drag_highlight(coverArt);
		coverArt.show();
		coverArt.set_from_pixbuf(Icons.DROP_ALBUM_PIXBUF);
		
		return false;
	}
	
	void drag_leave_signal(Gdk.DragContext context, uint time) {
		drag_unhighlight(coverArt);
		update_cover_art();
	}
	
	void drag_received(Gdk.DragContext context, int x, int y, Gtk.SelectionData data, uint info, uint timestamp) {
		if(lm.media_info.media == null || lm.media_info.media.isTemporary)
			return;
		
		bool success = true;
		
		foreach(string uri in data.get_uris()) {
			
			if(is_valid_image_type(uri) && lm.media_info.media != null) {
				message("Saving dragged album art as image\n");
				lm.save_album_locally(lm.media_info.media.rowid, uri);
			}
			else {
				message("Dragged album art is not valid image\n");
			}
			
			Gtk.drag_finish (context, success, false, timestamp);
			return;
		}
    }
	
	void update_sensitivities() {
		bool media_playing = lm.media_info.media != null;
		bool song_playing = media_playing && lm.media_info.media.mediatype == MediaType.SONG;
		//bool have_album_songs = (album_songs.get_table().size() > 1);
		bool have_artist_top_songs = (artist_top_songs.get_table().size() > 0);
		bool have_artist_top_albums = (artist_top_albums.get_table().size() > 0);
		var lastfm_elements_visible = lm.settings.getLastFMSessionKey() != "";
		
		loveMedia.set_visible(lastfm_elements_visible);
		banMedia.set_visible(lastfm_elements_visible);
		
		summary_box.set_visible(media_playing);
		artist_box.set_visible(song_playing && (have_artist_top_songs || have_artist_top_albums));
	}
	
	void update_contents() {
		Media m = lm.media_info.media;
		if(m == null) {
			return;
		}
		
		var title = "<span size=\"xx-large\">" + Markup.escape_text(m.title) + "</span>";
		var artist = ((m.artist != "" && m.artist != _("Unknown Artist")) ? ("<span size=\"x-large\">" + Markup.escape_text(m.artist) + "</span>") : "");
		var album = ((m.album != "" && m.album != _("Unknown Album")) ? ("<span size=\"large\">" + Markup.escape_text(m.album) + "</span>") : "");
		var year = ((m.year != 0) ? ("<span size =\"medium\">" + m.year.to_string() + "</span>") : "");
		
		meta_title.set_markup(title);
		meta_artist.set_markup(artist);
		meta_album.set_markup(album);
		meta_year.set_markup(year);
		rating.set_rating((int)m.rating);
		
		update_cover_art();
		update_album_songs();
		update_summary();
		update_lyrics();
		update_artist_section();
		
		update_sensitivities();
	}
	
	void update_cover_art() {
		if(lm.media_info.media == null) {
			return;
		}

		var coverart_pixbuf = lm.get_cover_album_art_from_key(lm.media_info.media.album_artist, lm.media_info.media.album);

		if(coverart_pixbuf != null) {
			coverArt.set_from_pixbuf(coverart_pixbuf);
		}
		else {
			coverArt.set_from_pixbuf(Icons.DEFAULT_ALBUM_ART_PIXBUF);
		}
	}
	
	void update_album_songs() {
		if(lm.media_info.media == null)
			return;
		
		Media m = lm.media_info.media;
		
		// Don't change the list if still in same album. Otherwise we lose temps.
		//var old = album_songs.get_media_from_index(0);
		//if(old != null && old.album_artist = m.album_artist && old.album == m.album)
		//	return;
		
		var results = new LinkedList<int>();
		lm.do_search (lm.media_ids(), out results, null, null, null, null,
	                  album_songs.get_hint(), "", m.album_artist, m.album);
	    
	    var table = new GLib.HashTable<int, Media>(null, null);
	    foreach(int i in results)
			table.set((int)table.size(), lm.media_from_id(i));
		
		// Try filling in missing songs w/ album info's tracks
		if(lm.media_info.album != null) {
			var fillers = new LinkedList<Media>();
			find_filler_songs(table, lm.media_info.album.tracks(), ref fillers);
			foreach(var media in fillers)
				table.set((int)table.size(), media);
		}
		
		album_songs.set_table(table);
		album_songs_holder.set_visible(table.size() > 1);
	}
	
	void find_filler_songs(HashTable<int, Media> in_library, Collection<Media> tests, ref LinkedList<Media> fillers) {
		foreach(Media test in tests) {
			bool found_match = false;
			
			for(int i = 0; i < in_library.size(); ++i) {
				Media s = in_library.get(i);
				if(test.track == s.track) {
					found_match = true;
					break;
				}
			}
			
			if(!found_match)
				fillers.add(test);
		}
	}
	
	void update_artist_section() {
		if(lm.media_info.media == null)
			return;
		
		artist_section_label.set_markup("<span size=\"x-large\"><b>" + Markup.escape_text(lm.media_info.media.artist) + "</b></span>");
		
		if(lm.media_info.artist != null) {
			var sim_artists = lm.media_info.artist.similarArtists();
			string label = "";
			foreach(var sim_art in sim_artists) {
				label += "<a href=\"" + sim_art.url.replace("&", "&amp;") + "\">" + Markup.escape_text(sim_art.name) + "</a>, ";
			}
			
			// Remove trailing ','
			if(label.length > 2)
				label = label.substring(0, label.length - 2);
			
			similar_artists.set_markup(("<b>%s: </b>").printf(_("Similar Artists")) + label);
			similar_artists.set_visible(sim_artists.size > 1);
			
			var tags = lm.media_info.artist.tags();
			label = "";
			foreach(var tag in tags) {
				label += "<a href=\"" + tag.url + "\">" + Markup.escape_text(tag.tag) + "</a>, ";
			}
			
			// Remove trailing ','
			if(label.length > 2)
				label = label.substring(0, label.length - 2);
			
			artist_tags.set_markup(("<b>%s: </b>").printf(_("Popular Tags")) + label);
			artist_tags.set_visible(tags.size > 1);
		}
		else {
			similar_artists.set_visible(false);
			artist_tags.set_visible(false);
		}
	}
	
	void size_allocate_signal(Allocation all) {
		update_lyrics();
	}
	
	void update_lyrics() {
		Media m = lm.media_info.media;
		if(m == null)
			return;
		
		if(m.mediatype == MediaType.SONG) {
			lyrics.set_label(m.lyrics);
			bool show_lyrics = m.lyrics != null && m.lyrics != "" && get_allocated_width() > 800;
			lyrics_holder.set_visible(show_lyrics);
		}
		else if(m.mediatype == MediaType.PODCAST) {
			lyrics.set_label(Markup.escape_text(m.comment));
			bool show_lyrics = m.comment != null && m.comment != "" && get_allocated_width() > 800;
			lyrics_holder.set_visible(show_lyrics);
		}
		else {
			lyrics_holder.set_visible(false);
		}
	}
	
	void update_summary() {
		bool have_title_info = 	lm.media_info.track != null && 
								lm.media_info.track.summary != null &&
								lm.media_info.track.summary != "";
		bool have_album_info = 	lm.media_info.album != null && 
								lm.media_info.album.summary != null &&
								lm.media_info.album.summary != "";
		bool have_artist_info = lm.media_info.artist != null && 
								lm.media_info.artist.summary != null &&
								lm.media_info.artist.summary != "";
		
		var whole_summary = "";
		if(have_title_info) {
			whole_summary += lm.media_info.track.summary;
		}
		if(have_album_info) {
			if(have_title_info)
				whole_summary += "\n\n";
			
			whole_summary += lm.media_info.album.summary;
		}
		if(have_artist_info) {
			if(have_title_info || have_album_info)
				whole_summary += "\n\n";
			
			whole_summary += lm.media_info.artist.summary;
		}
		
		summary_text.set_visible(have_title_info || have_album_info || have_artist_info);
		summary_text.set_markup(remove_html(whole_summary));
	}
	
	string remove_html(string s) {
		if(s == null || s == "")
			return "";
		
		string rv = s;
		Regex r;
		
		try {
			r = new Regex("<.*?>");
			rv = r.replace(s, s.length, 0, "");
		} catch(RegexError err) {
			warning("Regex error: %s. Now playing view will have bad text\n", err.message);
		}
		
		return rv;
	}
	
	public void show_video_mode() {
		int index = notebook.page_num (videoArea);
		notebook.set_current_page (index);
	}
	
	public void hide_video_mode() {
		int index = notebook.page_num (box);
		notebook.set_current_page (index);
	}
}
