/*-
 * Copyright (c) 2011-2012       Scott Ringwelski <sgringwe@mtu.edu>
 *
 * Originaly Written by Scott Ringwelski for BeatBox Music Player
 * BeatBox Music Player: http://www.launchpad.net/beat-box
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* Merely a place holder for multiple pieces of information regarding
 * the current media playing. Mostly here because of dependence. */

using Gtk;
using Gee;

public class BeatBox.SmartPlaylistEditor : Window {
	LibraryWindow lw;
	SmartPlaylist sp;
	
	VBox content;
	HBox padding;
	
	private  Label nameLabel;
	private Label rulesLabel;
	private Label optionsLabel;
	
	Granite.Widgets.HintedEntry nameEntry;
	ComboBoxText comboMatch;
	VBox vertQueries;
	Gee.ArrayList<SmartPlaylistEditorQuery> spQueries;
	Button addButton;
	CheckButton limitMedias;
	SpinButton mediaLimit;
	Button save;
	
	public signal void playlist_saved(SmartPlaylist sp);
	
	public SmartPlaylistEditor(LibraryWindow lw, SmartPlaylist sp) {
		this.lw = lw;
		this.sp = sp;
		
		this.title = _("Smart Playlist Editor");
		
		this.window_position = WindowPosition.CENTER;
		this.type_hint = Gdk.WindowTypeHint.DIALOG;
		this.set_modal(true);
		this.set_transient_for(lw);
		this.destroy_with_parent = true;
		
		content = new VBox(false, 10);
		padding = new HBox(false, 10);
		
		/* start out by creating all category labels */
		nameLabel = new Label(_("Name of Playlist"));
		rulesLabel = new Label(_("Rules"));
		optionsLabel = new Label(_("Options"));
		
		/* make them look good */
		nameLabel.xalign = 0.0f;
		rulesLabel.xalign = 0.0f;
		optionsLabel.xalign = 0.0f;
		nameLabel.set_markup("<b>%s</b>".printf(_("Name of Playlist")));
		rulesLabel.set_markup("<b>%s</b>".printf(_("Rules")));
		optionsLabel.set_markup("<b>%s</b>".printf(_("Options")));
		
		/* add the name entry */
		nameEntry = new Granite.Widgets.HintedEntry(_("Playlist Title"));
		if(sp.get_name() != "")
			nameEntry.set_text(sp.get_name());
		
		/* create match checkbox/combo combination */
		HBox matchBox = new HBox(false, 2);
		Label tMatch = new Label(_("Match"));
		comboMatch = new ComboBoxText();
		comboMatch.insert_text(0, _("any"));
		comboMatch.insert_text(1, _("all"));
		Label tOfTheFollowing = new Label(_("of the following:"));
		
		matchBox.pack_start(tMatch, false, false, 0);
		matchBox.pack_start(comboMatch, false, false, 0);
		matchBox.pack_start(tOfTheFollowing, false, false, 0);
		
		if(sp.conditional == SmartPlaylist.Conditional.ANY)
			comboMatch.set_active(0);
		else
			comboMatch.set_active(1);
		
		/* create rule list */
		spQueries = new Gee.ArrayList<SmartPlaylistEditorQuery>();
		vertQueries = new VBox(true, 2);
		foreach(SmartQuery q in sp.queries()) {
			SmartPlaylistEditorQuery speq = new SmartPlaylistEditorQuery(q);
			
			vertQueries.pack_start(speq, false, true, 1);
			spQueries.add(speq);
		}
		
		if(sp.queries().size == 0) {
			addRow();
		}
		
		addButton = new Button.with_label(_("Add"));
		vertQueries.pack_end(addButton, false, true, 1);
		addButton.clicked.connect(addButtonClick);
		
		/* create extra option: limiter */
		limitMedias = new CheckButton.with_label(_("Limit to"));
		mediaLimit = new SpinButton.with_range(0, 500, 10);
		Label limiterLabel = new Label(_("medias"));
		
		limitMedias.set_active(sp.limit);
		mediaLimit.set_value((double)sp.limit_amount);
		
		HBox limiterBox = new HBox(false, 2);
		limiterBox.pack_start(limitMedias, false, false, 0);
		limiterBox.pack_start(mediaLimit, false, false, 0);
		limiterBox.pack_start(limiterLabel, false, false, 0);
		
		/* add the Done button on bottom */
		HButtonBox bottomButtons = new HButtonBox();
		save = new Button.with_label(_("Done"));
		bottomButtons.set_layout(ButtonBoxStyle.END);
		bottomButtons.pack_end(save, false, false, 0);
		
		/* put it all together */
		content.pack_start(wrap_alignment(nameLabel, 10, 0, 0, 0), false, true, 0);
		content.pack_start(wrap_alignment(nameEntry, 0, 10, 0, 10), false, true, 0);
		content.pack_start(rulesLabel, false, true, 0);
		content.pack_start(wrap_alignment(matchBox, 0, 10, 0, 10) , false, true, 0);
		content.pack_start(wrap_alignment(vertQueries, 0, 10, 0, 10), false, true, 0);
		content.pack_start(optionsLabel, false, true, 0);
		content.pack_start(wrap_alignment(limiterBox, 0, 10, 0, 10), false, true, 0);
		content.pack_start(bottomButtons, false, false, 10);
		
		padding.pack_start(content, true, true, 10);
		
		add(padding);
		show_all();
		
		save.clicked.connect(saveClick);
		nameEntry.changed.connect(nameChanged);
	}
	
	public static Gtk.Alignment wrap_alignment (Gtk.Widget widget, int top, int right, int bottom, int left) {
		var alignment = new Gtk.Alignment(0.0f, 0.0f, 1.0f, 1.0f);
		alignment.top_padding = top;
		alignment.right_padding = right;
		alignment.bottom_padding = bottom;
		alignment.left_padding = left;
		
		alignment.add(widget);
		return alignment;
	}
	
	void nameChanged() {
		if(nameEntry.get_text() == "") {
			save.set_sensitive(false);
			return;
		}
		else {
			foreach(var p in lw.lm.smart_playlists()) {
				if((sp == null || sp.get_rowid() != p.get_rowid()) && nameEntry.get_text() == p.get_name()) {
					save.set_sensitive(false);
					return;
				}
			}
		}
		
		save.set_sensitive(true);
	}
	
	public void addRow() {
		SmartPlaylistEditorQuery speq = new SmartPlaylistEditorQuery(new SmartQuery());
		
		vertQueries.pack_start(speq, false, true, 1);
		spQueries.add(speq);
		
	}
	
	public virtual void addButtonClick() {
		addRow();
	}
	
	public virtual void saveClick() {
		sp.clearQueries();
		foreach(SmartPlaylistEditorQuery speq in spQueries) {
			if(speq.visible)
				sp.addQuery(speq.getQuery());
		}
		
		sp.set_name(nameEntry.text);
		sp.conditional = (SmartPlaylist.Conditional)comboMatch.get_active();
		sp.limit = limitMedias.get_active();
		sp.limit_amount = (int)mediaLimit.get_value();
		
		playlist_saved(sp);
		
		this.destroy();
	}
}

public class BeatBox.SmartPlaylistEditorQuery : HBox {
	SmartQuery q;
	
	ListStore field_model;
	ComboBox field;
	
	ListStore comparator_model;
	TreeModelFilter comparator_filter;
	ComboBox comparator;
	
	ListStore media_option_model;
	TreeModelFilter media_option_filter;
	ComboBox media_option;
	Entry entry;
	SpinButton spinbutton;
	
	Label units;
	Button removeButton;
	
	public signal void removed();
	
	public SmartPlaylistEditorQuery(SmartQuery q) {
		this.q = q;
		
		// Create drop downs
		field_model = new ListStore(2, typeof(int), typeof(string));
		field = new ComboBox.with_model(field_model);
		var cell = new CellRendererText();
		field.pack_start(cell, true);
		field.add_attribute(cell, "text", 1);
		field.set_id_column(0);
		field.set_entry_text_column(1);
		foreach(SmartQuery.Field field in SmartQuery.Field.all()) {
			TreeIter iter;
			field_model.append(out iter);
			field_model.set(iter, 0, (int)field, 1, field.to_string());
		}
		
		comparator_model = new ListStore(2, typeof(int), typeof(string));
		comparator_filter = new TreeModelFilter(comparator_model, null);
		comparator = new ComboBox.with_model(comparator_filter);
		cell = new CellRendererText();
		comparator.pack_start(cell, true);
		comparator.add_attribute(cell, "text", 1);
		comparator.set_id_column(0);
		comparator.set_entry_text_column(1);
		comparator_filter.set_visible_func(visible_func);
		foreach(SmartQuery.Comparator com in SmartQuery.Comparator.all()) {
			TreeIter iter;
			comparator_model.append(out iter);
			comparator_model.set(iter, 0, (int)com, 1, com.to_string());
		}
		
		entry = new Entry();
		spinbutton =  new SpinButton.with_range(0, 9999, 1);
		
		media_option_model = new ListStore(2, typeof(int), typeof(string));
		media_option_filter = new TreeModelFilter(media_option_model, null);
		media_option = new ComboBox.with_model(media_option_filter);
		cell = new CellRendererText();
		media_option.pack_start(cell, true);
		media_option.add_attribute(cell, "text", 1);
		media_option.set_id_column(0);
		media_option.set_entry_text_column(1);
		TreeIter iter;
		media_option_model.append(out iter);
		media_option_model.set(iter, 0, (int)MediaType.SONG, 1, MediaType.SONG.to_singular_string_up());
		media_option_model.append(out iter);
		media_option_model.set(iter, 0, (int)MediaType.PODCAST, 1, MediaType.PODCAST.to_singular_string_up());
		media_option_model.append(out iter);
		media_option_model.set(iter, 0, (int)MediaType.AUDIOBOOK, 1, MediaType.AUDIOBOOK.to_singular_string_up());
		media_option_model.append(out iter);
		media_option_model.set(iter, 0, (int)MediaType.STATION, 1, MediaType.STATION.to_singular_string_up());
		
		units = new Label("");
		removeButton = new Button.with_label(_("Remove"));
		
		field.set_active((int)q.field);
		comparator_filter.refilter();
		comparator_filter.foreach( (model, path, iter) => {
			int id;
			model.get(iter, 0, out id);
			
			if((SmartQuery.Comparator)id == q.comparator) {
				int index = int.parse(path.to_string());
				comparator.set_active(index);
				return true;
			}
			
			return false;
		});
		
		entry.set_no_show_all(true);
		media_option.set_no_show_all(true);
		spinbutton.set_no_show_all(true);
		
		comparator.set_size_request(160, -1);
		entry.set_size_request(200, -1);
		media_option.set_size_request(200, -1);
		spinbutton.set_size_request(200, -1);
		
		// Set the value
		if(q.field.is_string()) {
			entry.set_text(q.string_value);
			entry.show();
		}
		else if(q.field.is_type()) {
			media_option.set_active(q.int_value);
			media_option.show();
		}
		else if(q.field.is_int() || q.field.is_time()) {
			spinbutton.set_value(q.int_value);
			spinbutton.show();
		}
		
		pack_start(field, false, true, 0);
		pack_start(comparator, false ,true, 1);
		pack_start(entry, true, true, 1);
		pack_start(media_option, true, true, 1);
		pack_start(spinbutton, true, true, 1);
		pack_start(units, false, true, 1);
		pack_start(removeButton, false, true, 0);
		
		show_all();
		
		removeButton.clicked.connect(removeClicked);
		field.changed.connect(fieldChanged);
	}
	
	public SmartQuery getQuery() {
		SmartQuery rv = new SmartQuery();
		
		rv.field = (SmartQuery.Field)field.get_active();
		
		int id;
		TreeIter iter;
		comparator.get_active_iter(out iter);
		comparator_filter.get(iter, 0, out id);
		rv.comparator = (SmartQuery.Comparator)id;
		
		if(rv.field.is_string())
			rv.string_value = entry.text;
		else if(rv.field.is_type())
			rv.int_value = (MediaType)media_option.get_active();
		else
			rv.int_value = (int)spinbutton.value;
		
		return rv;
	}
	
	bool visible_func(TreeModel model, TreeIter iter) {
		bool rv = false;
		
		int id;
		model.get(iter, 0, out id);
		
		
		SmartQuery.Field cur_field = (SmartQuery.Field)field.get_active();
		switch (id) {
			case SmartQuery.Comparator.IS:
				rv = cur_field.is_string() || cur_field.is_type(); // strings, mediatype,
				break;
			case SmartQuery.Comparator.IS_EXACTLY:
				rv = cur_field.is_int() || cur_field.is_time(); // ints, times
				break;
			case SmartQuery.Comparator.IS_NOT:
				rv = cur_field.is_type(); //mediatype
				break;
			case SmartQuery.Comparator.IS_AT_LEAST:
			case SmartQuery.Comparator.IS_AT_MOST:
				rv = cur_field.is_int(); // ints
				break;
			case SmartQuery.Comparator.IS_WITHIN:
			case SmartQuery.Comparator.IS_BEFORE:
				rv = cur_field.is_time(); // times
				break;
			case SmartQuery.Comparator.CONTAINS:
			case SmartQuery.Comparator.DOES_NOT_CONTAIN:
				rv = cur_field.is_string(); // strings
				break;
			default:
				rv = false;
				break;
		}
		
		return rv;
	}
	
	public virtual void fieldChanged() {
		if(((SmartQuery.Field)field.get_active()).is_string()) {
			entry.show();
			media_option.hide();
			spinbutton.hide();
		}
		else if(((SmartQuery.Field)field.get_active()).is_type()) {
			entry.hide();
			spinbutton.hide();
			media_option.show();
			media_option.set_active(q.int_value);
		}
		else {
			spinbutton.show();
			entry.hide();
			media_option.hide();
		}
		
		comparator_filter.refilter();
		comparator.set_active(0);
		
		
		//helper for units
		if(((SmartQuery.Field)field.get_active()) == SmartQuery.Field.LENGTH) {
			units.set_text(_("seconds"));
			units.show();
		}
		else if(((SmartQuery.Field)field.get_active()).is_time()) {
			units.set_text(_("days ago"));
			units.show();
		}
		else if(((SmartQuery.Field)field.get_active()) == SmartQuery.Field.BITRATE) {
			units.set_text(_("kbps"));
			units.show();
		}
		else
			units.hide();
	}
	
	public virtual void removeClicked() {
		removed();
		hide();
	}
}
