/*-
 * Copyright (c) 2011-2012       Scott Ringwelski <sgringwe@mtu.edu>
 *
 * Originally Written by Scott Ringwelski for BeatBox Music Player
 * BeatBox Music Player: http://www.launchpad.net/beat-box
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gee;
using Gtk;

public class BeatBox.DuplicateViewWrapper : ViewWrapper {
	Collection<Media> all_media;
	HashMap<Media, Collection<Media>> dups;
	
	Gtk.HBox top_bar;
	Gtk.Button remove_checked;
	Gtk.ComboBoxText priority;
	
	GLib.CompareFunc<Media> first_priority;
	
	public DuplicateViewWrapper(LibraryWindow lw, Collection<Media> the_medias, TreeViewSetup tvs) {
		base(lw, new LinkedList<Media>(), tvs, -1);
		
		if(!(list_view is DuplicateList))
			critical("Duplicate View without Duplicate List. BeatBox will likely crash\n");
		
		top_bar = new HBox(false, 0);
		top_bar.margin = 6;
		remove_checked = new Gtk.Button.with_label(_("Remove Checked Items"));
		priority = new ComboBoxText();
		priority.append("manual", _("Manually select duplicates"));
		priority.append("bitrate", _("Keep highest bitrate"));
		priority.append("file", _("Keep largest file"));
		priority.set_active(0);
		first_priority = (GLib.CompareFunc)highest_bitrate;
		
		top_bar.pack_start(priority, false, false, 0);
		top_bar.pack_end(remove_checked, false, false, 0);
		pack_end(top_bar, false, true, 0);
		
		priority.changed.connect(priority_changed);
		remove_checked.clicked.connect(remove_checked_clicked);
		
		((DuplicateList)list_view).checked_changed.connect(checked_changed);
	}
	
	public void set_dups(HashMap<Media, Collection<Media>> dups) {
		this.dups = dups;
		
		var all_dups_list = new LinkedList<Media>();
		foreach(var m in dups.keys)
			all_dups_list.add(m);
		foreach(var list in dups.values)
			all_dups_list.add_all(list);
		
		set_media(all_dups_list);
		autocheck_medias();
	}
	
	public void update_visibilities () {
		top_bar.set_visible(dups.size > 0);
	}
	
	void autocheck_medias() {
		message("Autochecking items based on selected priority...\n");
		var dup_list = (DuplicateList)list_view;
		dup_list.clear_checked();
		
		if(priority.get_active() != 0) {
			foreach(var ent in dups.entries) {
				var all = new PriorityQueue<Media>(first_priority);
				all.offer(ent.key);
				foreach(var m in ent.value)
					all.offer(m);
				
				var chosen = all.peek();
				foreach(var m in all) {
					dup_list.set_checked(m, m != chosen);
				}
			}
		}
		
		// To update visual check marks, queue_draw
		queue_draw();
		checked_changed();
	}
	
	void priority_changed() {
		int a = priority.get_active();
		if(a == 1)
			first_priority = (GLib.CompareFunc)largest_file;
		else if(a == 2)
			first_priority = (GLib.CompareFunc)highest_bitrate;
		
		autocheck_medias();
	}
	
	void remove_checked_clicked() {
		var checked = ((DuplicateList)list_view).get_checked_medias();
		
		var rfd = new RemoveFilesDialog(lm.lw, checked, hint);
		rfd.remove_media.connect ( (delete_files) => {
			lm.remove_medias (checked, delete_files);
			
			// Close me
			lw.sideTree.removeItem(lw.sideTree.library_duplicates_iter);
			lw.sideTree.resetView();
		});
	}
	
	int largest_file(Media a, Media b) {
		if(a == null || b == null)
			return 0;
		
		int rv = (int)(a.file_size - b.file_size);
		return rv;
}
	
	int highest_bitrate(Media a, Media b) {
		if(a == null || b == null)
			return 0;
		
		int rv = (int)(a.bitrate - b.bitrate);
		return rv;
	}
	
	void checked_changed() {
		var checked = ((DuplicateList)list_view).get_checked_medias();
		remove_checked.set_sensitive(checked.size > 0);
	}
}
